/*
 *   senator - ASF video stream server
 *   Copyright (C) 2002  anonymous author
 *
 *
 *  senator is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  Senator is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *	
 *  You should have received a copy of the GNU General Public License
 *  along with senator; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 */
    
#ifndef CStreamHandler_h_included
#define CStreamHandler_h_included

#include "netclass/ncsocket.h"
#include "CASFFile.h"
#include "CLog.h"
#include "types.h"
#include "list.h"

enum packetType{
	streamHeader	= 0x4824,
	streamData	= 0x4424,
	streamEnd	= 0x4524
,
	streamClear	= 0x4324
};

enum mplayerStatus{
	requestFileHeader,
	requestStreamData
};

enum HTTPErrors{
	fileNotFound	= 404,
	accessForbidden	= 403,
	badRequest	= 400
,
	serverFull	= 503
};

#define	NUMBYTES	512

class CStreamHandler{
private:
	int		_ID;			// needed for the status page

	char		buf[NUMBYTES];		// client read buffer

	ncSocket	*_client;		// connection socket
	CASFFile	*_asfFile;		// file to read

	bool		_playing;
	
	unsigned short	_avarageDelay;

	std::string	_fileName;
	int		_sequence;		// we are what?
	int		_currentPacket;
	std::string	_clientVersion;

	std::string	_clientIP;

	void		sendProtocolPacketHeader(const packetType &t,const unsigned short &length,const unsigned short &part);
	void		sendFileHeader();
	void		sendStreamData(unsigned long offset);
	
	std::string 	getPragmaValue(const char* buf,const std::string &key);
	void		sendHTTPError(const HTTPErrors &e);
	void		sendHTTPResponse();
	
	void		sendStatusPage();
	
public:
	void		parseHTTPRequest(const char *buf);

	void		handleRequest();
	void		setConnectionID(const int &ID);
	int		getID();

	double		getPlayPercent();
	std::string	getPlayFileName();
	std::string	getClientVersion();
	std::string	getClientIP();
	double		getKBPerSec();

	CStreamHandler(ncSocket *io);
	~CStreamHandler();
};

#endif // CStreamHandler_h_included
