/*
 *   senator - ASF video stream server
 *   Copyright (C) 2002  anonymous author
 *
 *
 *  senator is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  Senator is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *	
 *  You should have received a copy of the GNU General Public License
 *  along with senator; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <signal.h>
#include <string>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <list>
  
#include "global.h"
#include "netclass/netclass.h"
#include "CLog.h"
#include "CStreamHandler.h"
#include "list.h"
#include "CConfigFile.h"

bool			running;		// while loopie
CConfigFile		*config;		// server config file

list <void *>		connectionList; 	// pointer vector to connection objects
ncSocketListener	*ncsl;			// server socket
ncMutex			updateConnectionList;	// mutex lock

extern char *itoa(int i, char *a, int r);

#define RUNNING_DIR	"/tmp"
#define LOCK_FILE	"/var/run/senator.lock"
#define LOG_FILE	"/var/log/senator.log"

void exitError(const std::string &errorText){
    logger->logError(errorText);
    logger->stop();
    running = false;
    exit(1);
}

void signal_handler(int sig){
    switch(sig) {
	case SIGINT:
	    logger->logInfo("interup signal: stopping server");
	    running = false;
	case SIGHUP:
	    logger->logInfo("hangup signal: stopping the server");
	    running = false;
	    break;
	case SIGTERM:
	    logger->logInfo("terminate signal stopping the server");
	    running = false;
	break;
    }
}

void daemonize(){
        int i,lfp;
        char str[10];
        if(getppid()==1)
	    exitError("already a daemong");
    
        i=fork();
        if (i<0) exit(1); // fork error 
        if (i>0) exit(0); // parent exits 

        // child (daemon) continues
        setsid(); // obtain a new process group 
    
        for (i=getdtablesize();i>=0;--i) close(i); // close all descriptors 
        //i=open("/dev/null",O_RDWR); 
        //dup(i);
        //dup(i); // handle standart I/O
	
        umask(027); // set newly created file permissions
        chdir(RUNNING_DIR); // change running directory 
        lfp=open(LOCK_FILE,O_RDWR|O_CREAT,0640);

        if (lfp<0){
    	    exitError("can not open lock file");
	}
        if (lockf(lfp,F_TLOCK,0)<0){
	    exitError("can not lock, server already running");
	}
    
        // first instance continues
        sprintf(str,"%d\n",getpid());
        write(lfp,str,strlen(str)); 	// record pid to lockfile

        signal(SIGCHLD,SIG_IGN); 		// ignore child
        signal(SIGTSTP,SIG_IGN); 		// ignore tty signals
	signal(SIGTTOU,SIG_IGN);
        signal(SIGTTIN,SIG_IGN);
        signal(SIGHUP,signal_handler); 	// catch hangup signal
        signal(SIGTERM,signal_handler); 	// catch kill signal
}

/*
 *  client callback
 *
 *  this will be called when ever a new client makes a connection
 *
 */
ncCallBackRetType handleClient(void *ptr){

    ncSocket *client = ((ncSocket *)ptr);
    if (client){
	client->setTimeout(0);	// create a non blocking server	
			
	CStreamHandler	*newHandler;
	newHandler	= new CStreamHandler(client);

	updateConnectionList.lock();
	connectionList.push_back((void*)newHandler);
	updateConnectionList.unlock();

	newHandler->handleRequest();

	updateConnectionList.lock();
	connectionList.remove((void*) newHandler);
	updateConnectionList.unlock();
		
	logger->logText("["+newHandler->getClientIP() + "] client disconnected");
	delete newHandler;
    }

	return (ncCallBackRetType)0;
}

/*
 *  server thread callback.
 *  waits for new client connections to handle
 *
 */
ncCallBackRetType serverFunc(void *ptr){
	logger->logText("waiting for connection");

	if (ncsl->startListening(handleClient,NC_THREADED, NC_DETACHED | NC_REUSEADDR) != NC_OK){
	    logger->logError("couldn't start the server, port already in use");
	    running = false;
	    return (ncCallBackRetType)0;
	}
	
	return (ncCallBackRetType)0;
}


void showUsage(){
	printf("options:\n");
        printf("\n");
        printf("   -c config.file     use configuration file config.file\n");
        printf("   -v                 verbosity flag\n");
        printf("   -d                 don't run as daemon\n");
        printf("   -h                 show this help text\n");
        printf("\n");
}

int main (int argc,char  *argv[]){
	netClass	nc;
	ncThread	serverThread;
				 
        const char    * configFileName	= "/etc/senator.conf";
        bool		verbosity	= false;
        int             i;
        const char      opts[] 		= "h?vdc:";
	bool		daemon		= true;

	running				= true;	// indicate that the server is in the air

	printf("\n");
	printf("Senator %s asf video streamer\n",VERSION);
        printf("Copyright (c) 2002, anonymous author\n");

// handle the server parameters		     
        while ( (i = getopt( argc, argv, opts)) != -1 ) {
    	    switch ( i ) {
    	    case 'c':
    		configFileName = optarg;
        	break;
            case 'v':
		verbosity = true;
    		break;
	    case 'd':
		daemon = false;
		break;
				    
	    default:
    	    case '?':
	    case 'h':
    	        showUsage();
		return 1;
	    }
        }

	logger = new CLog(LOG_FILE);
	logger->outputToConsole(verbosity);
	logger->start();

	config = new CConfigFile(configFileName);

	if (!config->parse()){
		logger->logError("can't find the config file or parse error");
		logger->logError("configFile : " + (std::string)configFileName);
		return 1;
	}

	char portNumber[20];
	itoa(config->portNumber,portNumber,10);
	logger->logInfo("running at port: "+(std::string)portNumber);

	if (daemon == true)
	    daemonize();	// from here you'r on your own kid.
	else 
	    logger->logInfo("server running in non daemon mode");

	logger->logText("installing signal handlers");
	signal(SIGINT, &signal_handler);

	ncsl = new ncSocketListener(config->portNumber,SOCKTYPE_TCPIP);

	// only required under win32 (initializes winsock)
	nc.initialize();

	logger->logInfo("starting the stream server");
	if (serverThread.start(serverFunc,(void*)&serverThread) == NC_FAILED){
		logger->logError("can not start the streamserver!");
		running = false;
	}

	while(running == true){
	    ncSleep(1);
	}
	
	serverThread.stop(0);
	logger->stop();

	return 0;
}
