/*
 *   senator - ASF video stream server
 *   Copyright (C) 2002  anonymous author
 *
 *
 *  senator is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  Senator is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *	
 *  You should have received a copy of the GNU General Public License
 *  along with senator; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 */

#include <stdio.h>
#include "CASFFile.h"

#pragma pack(1)

#include "guids.h"
#include "types.h"
#include "CConfigFile.h"
#include "global.h"

/* the chuck handling is quite lame...
 */

#define	READFILEHEADER		1
#define	READASFFILEHEADER	2

GUIDACTION _known_guids[] = {
	{&HeaderGUID,		"Header",		READASFFILEHEADER},
	{&FileHeaderGUID,	"File Header",		READFILEHEADER},
	{NULL, NULL, -1}
};

extern CConfigFile *config;

CASFFile::CASFFile(const std::string &fileName){
	this->_fileName = fileName;
	this->_opened	= false;
}

CASFFile::~CASFFile(){
    close();
}

/*
 *  checks if the file is there and opens is
 */
int CASFFile::open(){
	if (_opened == true)
		return 1;

	std::string path = config->rootDirectory +  this->_fileName;

#ifdef ASF_READER_DEBUG_OUTPUT
	printf("ASFDEBUG: full path = %s\n",path.c_str());
#endif
	if (strstr(_fileName.c_str(),"..") != NULL)	// hack protect
	    return ILLEGAL_REQUEST;

	if (strstr(_fileName.c_str(),"//") != NULL)	// hack protect
	    return ILLEGAL_REQUEST;

	if (strstr(_fileName.c_str(),"\\") != NULL)	// hack protect
	    return ILLEGAL_REQUEST;

	if (path[path.length() - 1] == '/' || path[path.length() - 1] == '\\')
	    return ILLEGAL_REQUEST;

	_fileHandler = fopen(path.c_str(),"rb");
	if (!_fileHandler)
		return FILE_NOT_FOUND;

#ifdef ASF_READER_DEBUG_OUTPUT	
	printf("ASFDEBUG: found the file, parsing\n");
#endif
	this->_opened = true;

	return FILE_OPENED;
}

int CASFFile::close(){
	if (!_opened) return 0;

	return fclose(_fileHandler);
}

int CASFFile::parseFile(){
	if (!_opened)
		return 0;
	
	fseek(_fileHandler,0,SEEK_SET);

#ifdef ASF_READER_DEBUG_OUTPUT	
	printf("ASFDEBUG: found the file, parsing\n");
#endif

	return parseChunk(&chunk);	// start with the first block
}

int	CASFFile::parseChunk(CHUNKSAVE *chunk){
	if (this->_opened == false) return 0;

	chunk->data = 0;

	// read a chunk GUID from the file
	if (fread(&chunk->guid, sizeof(chunk->guid), 1, _fileHandler) != 1)
		return 0;

	// read the length of the chunk
	if (fread(&chunk->len, sizeof(chunk->len), 1, _fileHandler) != 1)
		return 0;

	// search the known GUIDs to find the corresponding handler function

	int result,retval = 0;
	result	= executeChunckFunction(chunk);

	// no handler found for the block... skip the data in the file
	if (result == 0){
		unsigned long datalen = (unsigned long) (chunk->len - 24);
		if ((chunk->data = malloc(datalen)) == NULL)
			return 0;
		if (fread(chunk->data, datalen, 1, _fileHandler) != 1){
			free(chunk->data);
			chunk->data = NULL;
			return 0;
		}
		retval = 1;
	} else {
		retval = 1;
	}
	return retval;
}

/*
 *  debug function to print the chunk GUID to the screen
 */
void CASFFile::printChunkData(const GUID &guid){
        static char     guid_str[20];
        sprintf(guid_str, "(%2x%2x-%2x-%2x-%2x%2x-%2x%2x%2x%2x%2x%2x)",
	        guid.Data1 >> 8,
	        guid.Data1 & 255,
	        guid.Data2,
		guid.Data3,
                guid.Data4[0],
                guid.Data4[1],
                guid.Data4[2],
                guid.Data4[3],
                guid.Data4[4],
                guid.Data4[5],
                guid.Data4[6],
                guid.Data4[7]);
        printf("ASFDEBUG: file offset: %d, GUID : %s\n",ftell(_fileHandler),guid_str);
}

int CASFFile::executeChunckFunction(CHUNKSAVE *chunk){
	int			retval = 0;
	GUIDACTION	*curr;
	bool		found = false;

	GUID		*guid = &chunk->guid;

#ifdef ASF_READER_DEBUG_OUTPUT
	printf("ASFDEBUG: searching for chunk handler\n");
	printChunkData(chunk->guid);
#endif	

	curr = _known_guids;
	while (curr->guid != NULL) {
		if (memcmp(curr->guid, (void*)guid, sizeof(GUID)) == 0) {
			switch(curr->fn){
			case READFILEHEADER:
				retval = readFileHeader(chunk);
				break;
			case READASFFILEHEADER:
				retval = readASFFileHeader(chunk);
				break;
			default:
				break;
			}
			found = true;
			break;
		}
		curr++;
	}

	return retval;
}

int CASFFile::readASFFileHeader(CHUNKSAVE *chunk){

	struct header_st {
		ASF_int32 	numchunks;
		ASF_short	unknown;
	} header;

	int 		i;
	int 		savecount = 0;
	CHUNKSAVE 	*head = NULL, *tail = NULL;
	ASF_uint64 	bytesremoved = 0;
	
	this->subchunk	= NULL;
	_asfHeaderSize	= 0;
	_headerCount	= 0;

	// read the amount for chunks header from the source file
	if (fread(&header, sizeof(header), 1, _fileHandler) != 1)
		return 0;

	_headerCount = header.numchunks;

#ifdef	ASF_READER_DEBUG_OUTPUT
	printf("ASFDEBUG: found the file header block... total chunks left %d\n",_headerCount);
#endif
	for (i = 0; i < header.numchunks; i++) {		// for each chunk
		if (subchunk == NULL) {
			if ((subchunk = (chunk_save_st *)malloc(sizeof(CHUNKSAVE))) == NULL)
				return 0;
		}

#ifdef	ASF_READER_DEBUG_OUTPUT
    		printf("ASFDEBUG: parsing header %d of %d\n\n",i,_headerCount);
#endif
		if (!this->parseChunk(subchunk))	// see if we have a function that can parse this chunk
			return 0;
		
		this->_asfHeaderSize+=(unsigned short)subchunk->len;

		if (subchunk->data != NULL) {		// yep :)
			if (tail == NULL)
				head = subchunk;
			else
				tail->next = subchunk;
			
			subchunk->next = NULL;
			tail = subchunk;

			savecount++;
		} else {
			bytesremoved += subchunk->len;
		}
	}

	_asfHeaderSize+=sizeof(FILEHEADER);
	subchunk = head;
	
	return 1;
}

/*
 * read the ASF file header into the memory
 *
 */ 
int CASFFile::readFileHeader(CHUNKSAVE *chunk){

	unsigned long datalen = (unsigned long) (chunk->len - 24);

#ifdef ASF_READER_DEBUG_OUTPUT
	printf("ASFDEBUG: found the ASF data header\n");
#endif

	if ((chunk->data = malloc(datalen)) == NULL)
		return 0;

	if (fread(chunk->data, datalen, 1, _fileHandler) != 1){
		free(chunk->data);
		chunk->data = NULL;
		return 0;
	}

	this->_fileHeader = (FILEHEADER *)chunk->data;

#ifdef ASF_READER_DEBUG_OUTPUT
	printf("ASFDEBUG: ASF data header\n");
        printf("          _fileHeader.filesize = %d\n", _fileHeader->filesize);
        printf("          _fileHeader.fileCreateTime = %i\n", _fileHeader->fileCreateTime);
        printf("          _fileHeader.numPackets = %d\n", _fileHeader->numPackets);
        printf("          _fileHeader.timeAtEnd = %d\n",_fileHeader->timeAtEnd);
        printf("          _fileHeader.playDuration = %d\n",_fileHeader->playDuration);
        printf("          _fileHeader.timeAtStart = %d\n",_fileHeader->timeAtStart);
        printf("          _fileHeader.unknown1 = %d\n",_fileHeader->unknown1);
        printf("          _fileHeader.unknown2 = %d\n",_fileHeader->unknown2);
        printf("          _fileHeader.packetSize = %d\n",_fileHeader->packetSize);
        printf("          _fileHeader.packetSize2 = %d\n",_fileHeader->packetSize2);
        printf("          _fileHeader.uncompressedSize = %d\n",_fileHeader->uncompressedSize);
#endif

	return 1;
}

/*
 *  getFileHeader
 *
 *  return the fileheader of an ASF file
 */
void CASFFile::getASFHeader(void *buffer){
	if (!_opened) return;
	
 	fseek(_fileHandler,0,SEEK_SET);	// jump to the start
	fread(buffer,this->_asfHeaderSize,1,this->_fileHandler);
}

/*
 *  getASFHeaderSize
 *
 *  seek the correct frame in the movie and return it to the caller
 */
int CASFFile::getASFHeaderSize(){
	return this->_asfHeaderSize;
}

/*
 *  getFileData
 *
 *  seek the correct frame in the movie and return it to the caller
 */
void CASFFile::getFrame(void *buffer,int packetOffset){
	if (!_opened) return;
	long offset = this->_asfHeaderSize + (packetOffset * this->_fileHeader->packetSize);

	fseek(_fileHandler,offset,SEEK_SET);
	fread(buffer,this->_fileHeader->packetSize,1,_fileHandler);
}


/*
 *  getFileHeader
 *
 *  returns the FILEHEADER structure
 */
void CASFFile::getFileHeader(FILEHEADER *buffer){
	memcpy(buffer,this->_fileHeader,sizeof(FILEHEADER));
}
